
/*  System of linear equations. Ax = b.
 *  Visualize matrix A, study sparsity and type of 
 *  constraints.
 *
 *  Author: Vijay C Patil.
 *  Date: August 2008
 *  */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include <string.h>
#include <GL/glut.h>
#include <glpk.h>
#include "matrix.h"

/* Global objects. */
int row_cnt = 0, col_cnt = 0;
double cell_width, cell_height;
double mat_width, mat_height; 
glp_prob * lp = NULL;

/* Entry point. */
int main(int argc, char * argv[])
{
	char win_title[64];
	if(argc != 2) 
		print_usage();

	/* Lets initialize GLUT. */
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);

	/* Specify window properties and create window. */
	glutInitWindowSize(WIDTH, HEIGHT);
	glutInitWindowPosition(100, 100);
	strcpy(win_title, "Matrix Visualisation: "); 
	strcat(win_title, argv[1]);
	glutCreateWindow(win_title);

	/* Register functions. */
	glutReshapeFunc(win_reshape);
	glutDisplayFunc(draw_matrix);

	glpk_init(argv[1]);

	/* Listen and process events in a loop. */
	glutMainLoop();

	return 0;
}

void print_usage(void)
{
	printf("\nUsage: matrix filename\n");
	printf("Where filename is MathProg model file name.\n");
	printf("Color of the cell depends on the its value.\n");
	printf("BLUE 	= Positive value.\n");
	printf("RED 	= Negative value.\n");
	printf("BLACK 	= Zero.\n\n");
	exit(-1);
}

void win_reshape(int win_width, int win_height)
{

	mat_width = win_width - 2 * OFFSET;
	mat_height = win_height - 2 * OFFSET;

	/* Obj. func. coeff. will be an additional row. */
	/* UB and LB will be an additional columns. */
	col_cnt = glp_get_num_cols(lp);
	row_cnt = glp_get_num_rows(lp);
	if(col_cnt == 0 || row_cnt == 0) {
		printf("Warning: Model has zero rows or zero columns.\n");
		exit(-1);
	}

	cell_width = mat_width/(double)col_cnt;
	cell_height = mat_height/(double)row_cnt;

	if(cell_width < cell_height) {
		cell_height = cell_width;
		mat_height = cell_height * row_cnt;
	} else {
		cell_width = cell_height;
		mat_width = cell_width * col_cnt;
	}

	glClearColor(1.0, 1.0, 1.0, 0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	set_window(0.0, win_width, win_height, 0.0);
	set_viewport(0.0, win_width, 0.0, win_height);
	glutPostRedisplay();

}

/* Init and load matrix. */
void glpk_init(char * mod_filename)
{
	lp = lpx_read_model(mod_filename, NULL, NULL);
	if(lp == NULL) {
		printf("Error: GLPK could not read model file %s\n", mod_filename);
		exit(-1);
	}
}

/* Viewport */
void set_viewport(float left, float right, float bottom, float top)
{
	glViewport(left, bottom, right-left, top-bottom);
}

/* World window. */
void set_window(float left, float right, float bottom, float top)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(left, right, bottom, top);
}

