
/* Author: Vijay C. Patil. */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <GL/glut.h>
#include "matrix.h"

/* Given row number and column number of a matrix cell
 * draw it at correct position, and color it properly.
 */
void draw_cell(int row_ind, int col_ind, double value)
{
	//printf("r=%d, c=%d, Cell value = %lf\n", row_ind, col_ind, value);

	if(value > 0.0)
		glColor3f(0.0, 0.0, 1.0); /* blue */
	else if(value < 0.0)
		glColor3f(1.0, 0.0, 0.0); /* red */
	else
		assert(0);

	glBegin(GL_QUADS);
		double top_left_x = OFFSET + cell_width * (col_ind-1);
		double top_left_y = OFFSET + cell_height * (row_ind-1);

		glVertex2f(top_left_x, top_left_y);
		glVertex2f(top_left_x, top_left_y + cell_height);
		glVertex2f(top_left_x + cell_width, top_left_y + cell_height);
		glVertex2f(top_left_x + cell_width, top_left_y);

	glEnd();
}

/* Display callback function. */
void draw_matrix(void)
{
	int ci, row_ind, row_nz_cnt;
	int * ind = NULL;
	double * val = NULL;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	/* BLACK background. */
	glColor3f(0.0, 0.0, 0.0);
	glBegin(GL_QUADS);
		// bottom left
		glVertex2f(OFFSET, OFFSET + mat_height);
		// top left
		glVertex2f(OFFSET, OFFSET);
		// top right 
		glVertex2f(OFFSET + mat_width, OFFSET);
		// bottom right 
		glVertex2f(OFFSET + mat_width, OFFSET + mat_height);
	glEnd();

	row_cnt = glp_get_num_rows(lp);
	if(row_cnt == 0) 
		printf("Warning: Zero rows in the model.\n");

	/* Ignore objective function ? */
	for(row_ind = 1; row_ind <= row_cnt; row_ind++) {

		row_nz_cnt = glp_get_mat_row(lp, row_ind, NULL, NULL); 
		if(row_nz_cnt == 0)
			continue;

		//printf("row_ind = %d, row_nz_cnt = %d\n", row_ind, row_nz_cnt);

		ind = (int*) calloc(row_nz_cnt+1, sizeof(int)); 
		val = (double*) calloc(row_nz_cnt+1, sizeof(double)); 

		glp_get_mat_row(lp, row_ind, ind, val); 

		/* Draw all non-zero cells i.e. matrix coefficients. */
		for(ci = 1; ci <= row_nz_cnt; ci++)
			draw_cell(row_ind, ind[ci], val[ci]);

		free(ind); ind = NULL;
		free(val); val = NULL;
	}

	glFlush();
	glutSwapBuffers();
}

